
/* Chrysalide - Outil d'analyse de fichiers binaires
 * cse.c - manipulation des types classes / structures / énumérations
 *
 * Copyright (C) 2012-2017 Cyrille Bagard
 *
 *  This file is part of Chrysalide.
 *
 *  Chrysalide is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 3 of the License, or
 *  (at your option) any later version.
 *
 *  Chrysalide is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with Foobar.  If not, see <http://www.gnu.org/licenses/>.
 */


#include "cse.h"


#include <string.h>


#include "cse-int.h"



/* Initialise la classe des types classe ou assimilés. */
static void g_class_enum_type_class_init(GClassEnumTypeClass *);

/* Initialise l'instance d'un type classe ou assimilé. */
static void g_class_enum_type_init(GClassEnumType *);

/* Crée un copie d'un type existant. */
static GDataType *g_class_enum_type_dup(const GClassEnumType *);



/* Indique le type défini pour un type classe ou assimilé. */
G_DEFINE_TYPE(GClassEnumType, g_class_enum_type, G_TYPE_DATA_TYPE);


/******************************************************************************
*                                                                             *
*  Paramètres  : klass = classe à initialiser.                                *
*                                                                             *
*  Description : Initialise la classe des types classe ou assimilés.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_class_enum_type_class_init(GClassEnumTypeClass *klass)
{

}


/******************************************************************************
*                                                                             *
*  Paramètres  : type = instance à initialiser.                               *
*                                                                             *
*  Description : Initialise l'instance d'un type classe ou assimilé.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static void g_class_enum_type_init(GClassEnumType *type)
{
    GDataType *data_type;                   /* Version basique             */

    data_type = G_DATA_TYPE(type);

    data_type->dup = (type_dup_fc)g_class_enum_type_dup;
    data_type->to_string = (type_to_string_fc)g_class_enum_type_to_string;
    //data_type->output = (output_type_fc)g_class_enum_type_output;

}


/******************************************************************************
*                                                                             *
*  Paramètres  : type = type de structure à représenter.                      *
*                name = désignation humaine du type.                          *
*                                                                             *
*  Description : Crée une représentation de classe, structure ou énumération. *
*                                                                             *
*  Retour      : Adresse de la structure mise en place.                       *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

GDataType *g_class_enum_type_new(ClassEnumType type, char *name)
{
    GClassEnumType *result;                 /* Structure à retourner       */

    result = g_object_new(G_TYPE_CLASS_ENUM_TYPE, NULL);

    result->type = type;
    result->name = name;

    return G_DATA_TYPE(result);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : type = type à dupliquer.                                     *
*                                                                             *
*  Description : Crée un copie d'un type existant.                            *
*                                                                             *
*  Retour      : Nouvelle instance de type identique à celle fournie.         *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

static GDataType *g_class_enum_type_dup(const GClassEnumType *type)
{
    return g_class_enum_type_new(type->type, type->name);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : type = type à convertir.                                     *
*                                                                             *
*  Description : Décrit le type fourni sous forme de caractères.              *
*                                                                             *
*  Retour      : Chaîne à libérer de la mémoire après usage.                  *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/

char *g_class_enum_type_to_string(const GClassEnumType *type)
{
    return strdup(type->name);

}


/******************************************************************************
*                                                                             *
*  Paramètres  : type   = type à afficher.                                    *
*                lang   = langage à utiliser pour la sortie humaine.          *
*                buffer = tampon mis à disposition pour la sortie.            *
*                info   = nature du cadre de destination.                     *
*                full   = besoin de descriptions étendues ?                   *
*                                                                             *
*  Description : Procède à l'impression de la description d'un type.          *
*                                                                             *
*  Retour      : -                                                            *
*                                                                             *
*  Remarques   : -                                                            *
*                                                                             *
******************************************************************************/
#if 0
void g_class_enum_type_output(const GClassEnumType *type, GLangOutput *lang, GBufferLine *line, bool info, bool full)
{
    g_buffer_line_append_text(line, BLC_LAST_USED, type->name, strlen(type->name),
                              info ? RTT_COMMENT : RTT_RAW, NULL);

}
#endif
